#!/usr/bin/env python

from sys import argv, exit, stdout, stderr
from os import path
import metadata_reader


image_width_pixels = int(400)


argc = len(argv)

min_args = 2

if (argc < min_args) :
    stderr.write("\nUsage: " + argv[0]
                 + " metadata_file_1 [ metadata_file 2 ... ]\n\n")
    exit(1)


argptr = int(1)


metadata_file_list = list()

while (argptr < argc) :
    metadata_file_list.append(str(argv[argptr]))
    argptr += 1

num_files = len(metadata_file_list)


## Write the header

stdout.write("<html>\n")
stdout.write("<head><title>Diagnostic Tool Slices</title></head>\n")
stdout.write("<body>\n\n")

stdout.write("<p>Click on the images below for detailed information.</p>\n\n")

stdout.write("<table border=\"1\" cellpadding=\"5\">\n")

stdout.write("  <tr>\n")
stdout.write("    <th>Time</th>\n")
stdout.write("    <th>Datafile</th>\n")
stdout.write("    <th>Estimated Flame Pos</th>\n")
stdout.write("    <th>Flame Image <em>(Click image for additional "
             + "slices.)</em>" + "</th>\n")
stdout.write("  </tr>\n")


## Run through the files 

for file_index in range(num_files) :

    metadata_file = metadata_file_list[file_index]

    file_md, dataset_md_list = metadata_reader.parse_metadata(metadata_file)

    num_datasets = len(dataset_md_list)

    datafile_basename = path.split(file_md.get_group_name())[1]

    svg_file = datafile_basename + "_slice.svg"
    
    sim_time = file_md["Time"]

    flame_pos = file_md["Flame_Pos"]

    slice_min_coords = file_md["Grid_Min_Coords"]
    slice_max_coords = file_md["Grid_Max_Coords"]

    slice_width = slice_max_coords[0] - slice_min_coords[0]
    slice_height = slice_max_coords[1] - slice_min_coords[1]

    aspect_ratio_w_h = slice_width / slice_height

    image_height_pixels = int(float(image_width_pixels) / aspect_ratio_w_h)

    ## Find the flame image

    flame_md = None

    dataset_index = int(0)

    while ((flame_md is None) and (dataset_index < num_datasets)) :

        test_md = dataset_md_list[dataset_index]

        if (test_md.get_group_name() == "flam") :
            flame_md = test_md
        else :
            dataset_index += 1

    if (flame_md is None) :
        stderr.write("\nError: Dataset flam not found\n\n")
        raise RuntimeError("Dataset flam not found")

    xy_image_md = None
    xy_image_file = None

    ## Look for the image file

    image_md_list = flame_md.get_keyword_group_list(group_type="Image")

    num_images = len(image_md_list)

    image_index = 0

    while ((image_index < num_images) and (xy_image_md is None)) :

        image_md = image_md_list[image_index]

        plane_id = image_md.get_group_name()

        if (plane_id == "xy") :
            xy_image_md = image_md
        else :
            image_index += 1

    ## See if we should use the dataset instead

    if (xy_image_md is not None) :

        xy_image_file = xy_image_md["File"]

    else :

        data_md_list = flame_md.get_keyword_group_list(group_type="Data")

        num_images = len(data_md_list)

        while ((image_index < num_images) and (xy_image_md is None)) :

            image_md = data_md_list[image_index]

            plane_id = image_md.get_group_name()

            if (plane_id == "xy") :
                xy_image_id = image_md
            else :
                image_index += 1

        if (xy_image_id is None) :
            stderr.write("\nError: Dataset flam xy slice image not found\n\n")
            raise RuntimeError("Dataset flam xy slice image not found")

        xy_image_file = xy_image_md["File"] + ".png"

    stdout.write("  <tr>\n")
    stdout.write("    <td align=\"center\" valign=\"middle\">"
                 + str(sim_time) + "</td>\n")
    stdout.write("    <td align=\"center\" valign=\"middle\"><a href=\""
                 + svg_file + "\">" + datafile_basename + "</a></td>\n")
    stdout.write("    <td align=\"center\" valign=\"middle\">"
                 + str(flame_pos) + "</td>\n")
    stdout.write("    <td align=\"center\" valign=\"middle\"><a href=\""
                 + svg_file + "\"><img src=\"" + xy_image_file
                 + "\" width=\"" + str(image_width_pixels) + "\" height=\""
                 + str(image_height_pixels) + "\"></a></td>\n")
    stdout.write("  </tr>\n")

## Close the webpage

stdout.write("</table>\n")

stdout.write("</body>\n")
stdout.write("</html>\n")

stdout.write("\n")
