// C++ header file flame_dist.hpp

/*
  By Nathan C. Hearn
     May 6, 2008

  Distribution of material within flame.
*/


#ifndef FLAME_DIST_HPP
#define FLAME_DIST_HPP


#include <vector>
#include "quickflash_file_datafile.hpp"


class FlameDistribution
  {
  public :

    FlameDistribution();

    FlameDistribution(const QuickFlash::File::DataFile & df, 
		      const unsigned int search_axis,
		      const double bin_array_min_pos,
		      const unsigned int bin_count, 
		      const double sample_bin_width,
		      const double min_flame_value, 
		      const double max_flame_value);

    FlameDistribution(const FlameDistribution & source);

    ~FlameDistribution() { }

    FlameDistribution & operator=(const FlameDistribution & source)
      {
      reset(source);
      return *this;
      }

    void reset();

    void reset(const QuickFlash::File::DataFile & df, 
	       const unsigned int search_axis,
	       const double bin_array_min_pos,
	       const unsigned int bin_count, const double sample_bin_width,
	       const double min_flame_value, const double max_flame_value);

    void reset(const FlameDistribution & source);

    double get_total_mass() const { return total_mass; }
    double get_total_burned_mass() const { return total_burned_mass; }
    double get_total_unburned_mass() const { return total_unburned_mass; }

    double get_array_min_coord() const { return bin_array_min_coord; }
    double get_array_max_coord() const { return bin_array_max_coord; }

    unsigned int get_num_bins() const { return num_bins; }

    const std::vector<double> & get_bin_centers() const { return bin_centers; }
    const std::vector<double> & get_bin_volumes() const { return bin_volumes; }

    const std::vector<double> & get_mass_bins_total() const 
      { return mass_bins_total; }

    const std::vector<double> & get_dens_bins_total_mean() const
      { return dens_bins_total_mean; }

    const std::vector<double> & get_dens_bins_total_dispersion() const
      { return dens_bins_total_dispersion; }

    const std::vector<double> & get_mass_bins_burned() const 
      { return mass_bins_burned; }

    const std::vector<double> & get_dens_bins_burned_mean() const
      { return dens_bins_burned_mean; }

    const std::vector<double> & get_dens_bins_burned_dispersion() const
      { return dens_bins_burned_dispersion; }

    const std::vector<double> & get_mass_bins_unburned() const 
      { return mass_bins_unburned; }

    const std::vector<double> & get_dens_bins_unburned_mean() const
      { return dens_bins_unburned_mean; }

    const std::vector<double> & get_dens_bins_unburned_dispersion() const
      { return dens_bins_unburned_dispersion; }

    const std::vector<double> & get_mass_fract_bins_burned() const
      { return mass_fract_bins_burned; }

    double get_center_pos_mass_fract() const { return mass_fract_center_pos; }
    double get_center_pos_mass() const { return mass_center_pos; }

    double get_flame_min_pos() const { return flame_min_pos; }
    double get_flame_max_pos() const { return flame_max_pos; }

  private :

    int find_flame_center_mass_fract();
    int find_flame_center_mass();
    int find_flame_extremes(const double min_flame_value, 
			    const double max_flame_value);

  private :

    double total_mass;
    double total_burned_mass;
    double total_unburned_mass;

    double bin_array_min_coord;
    double bin_array_max_coord;

    unsigned int num_bins;

    double bin_width;

    std::vector<double> bin_centers;
    std::vector<double> bin_volumes;

    std::vector<double> mass_bins_total;
    std::vector<double> dens_bins_total_mean;
    std::vector<double> dens_bins_total_dispersion;

    std::vector<double> mass_bins_burned;
    std::vector<double> dens_bins_burned_mean;
    std::vector<double> dens_bins_burned_dispersion;

    std::vector<double> mass_bins_unburned;
    std::vector<double> dens_bins_unburned_mean;
    std::vector<double> dens_bins_unburned_dispersion;

    std::vector<double> mass_fract_bins_burned;

    double mass_fract_center_pos;
    double mass_center_pos;

    double flame_min_pos;
    double flame_max_pos;
  };


#endif  // FLAME_DIST_HPP
