// C++ header file flash_resample.hpp

/*
  By Nathan C. Hearn
     May 21, 2007

  Resampling routines for Flash.
*/


#ifndef FLASH_RESAMPLE_HPP
#define FLASH_RESAMPLE_HPP


#ifndef SEPARATE_FROM_FLASH
#include "mangle_names.h"
#else
#define FTOC(x) x
#endif  // SEPARATE_FROM_FLASH


extern "C"
{
//! Load Flash2/3 HDF5 file
//! Returns zero on success, negative otherwise
//! WARNING: Must be called successfully before using any functions below
//! WARNING: Subsequent calls will close any previously open datasets or files
int FTOC(quickflash_init_resample)(const char file_name[]);

//! Load dataset
//! Returns local dataset index on success, negative otherwise
int FTOC(quickflash_add_dataset)(const char dataset_name[]);

//! Check position
//! Returns true if in bounds, false otherwise
bool FTOC(quickflash_in_bounds)(const double & x, const double & y,
			      const double & z);

//! Get number of mesh dimensions
//! Returns dimension count on success, negative otherwise
int FTOC(quickflash_dims)();

//! Get the min volume boundary (lower corner of mesh)
//! Returns zero on success, negative otherwise
int FTOC(quickflash_min_bounds)(double & x, double & y, double & z);

//! Get the max volume boundary (upper corner of mesh)
//! Returns zero on success, negative otherwise
int FTOC(quickflash_max_bounds)(double & x, double & y, double & z);

//! Get the block and cell indexes
//! Returns zero on success, positive if out of bounds, negative otherwise
//! Note: block_index and cell_index are not valid if out of bounds
int FTOC(quickflash_get_block_cell)(const double & x, const double & y, 
				  const double & z, int & block_index,
				  int & cell_index);

//! Get the value of the selected dataset at specified block and cell indexes
//! Returns zero on success, negative otherwise
//! Note: dataset_index is the value returned by quickflash_add_dataset
int FTOC(quickflash_get_value)(const int & dataset_index, 
			     const int & block_index, 
			     const int & cell_index, double & value);

//! Clean caches and close data file
//! Returns zero on success, negative otherwise
//! WARNING: Call only when finished with ALL init_block-type loops
int FTOC(quickflash_clean_up)();

}  // End extern "C"


#endif  // FLASH_RESAMPLE_HPP
