// C++ program file grid_fft.cpp

/*
  By Nathan C. Hearn
     October 30, 2007

  FFT-related utilities.
*/


#include "grid_fft.hpp"
#include <vector>
#include "quickflash_block_blockdata.hpp"
#include "fft.hpp"



int get_power_spectra(const std::vector<double> & data_vector,
		      std::vector<double> & power_vector)
  {
  int ret_val = -1;  // Error state

  std::vector< std::complex<double> > trans_data;

  FFT fft;

  if (fft.compute_fft(data_vector, trans_data) >= 0)
    ret_val = get_power(trans_data, power_vector);

  return ret_val;
  }


int get_power_spectra_row(const QuickFlash::Block::BlockData<double> & data_grid,
			  const unsigned int row_index,
			  std::vector<double> & power_vector)
  {
  int ret_val = -1;

  const std::vector<unsigned int> & grid_dims = data_grid.get_block_dims();

  if (grid_dims.size() != 2)
    throw QuickFlash::Except("Only 2D grids supported at this time", __FILE__,
			   __LINE__);

  const unsigned int num_rows = grid_dims[0];
  const unsigned int num_cols = grid_dims[1];

  if (num_rows > row_index)
    {
    // Extract the row

    std::vector<double> row_data(num_cols);

    std::vector<unsigned int> grid_index(2);

    grid_index[0] = row_index;

    for (unsigned int index = 0; index < num_cols; index++)
      {
      grid_index[1] = index;
      row_data[index] = data_grid[grid_index];
      }

    // Perform the fft

    ret_val = get_power_spectra(row_data, power_vector);
    }

  return ret_val;
  }


int get_power_spectra_row(const QuickFlash::Block::BlockData<double> & data_grid,
			  QuickFlash::Block::BlockData<double> & power_grid)
  {
  int ret_val = -1;

  const std::vector<unsigned int> & grid_dims = data_grid.get_block_dims();

  if (grid_dims.size() != 2)
    throw QuickFlash::Except("Only 2D grids supported at this time", __FILE__,
			   __LINE__);

  const unsigned int num_rows = grid_dims[0];
  const unsigned int num_cols = grid_dims[1];

  const unsigned int num_power_cols = get_power_vector_size(num_cols);

  std::vector<unsigned int> power_grid_dims(2);

  power_grid_dims[0] = num_rows;
  power_grid_dims[1] = num_power_cols;

  power_grid.reset(power_grid_dims);

  std::vector<double> row_data(num_cols);
  std::vector<double> power_vector(num_power_cols);

  std::vector<unsigned int> grid_index(2);

  ret_val = 0;  // OK so far ...

  for (unsigned int row_index = 0; row_index < num_rows; row_index++)
    {
    // Extract the row

    grid_index[0] = row_index;

    for (unsigned int col_index = 0; col_index < num_cols; col_index++)
      {
      grid_index[1] = col_index;
      row_data[col_index] = data_grid[grid_index];
      }

    // Perform the fft

    if (get_power_spectra(row_data, power_vector) >= 0)
      {
      for (unsigned int col_index = 0; col_index < num_power_cols; col_index++)
	{
	grid_index[1] = col_index;
	power_grid[grid_index] = power_vector[col_index];
	}
      }
    else
      ret_val = -1;  // Error state
    }

  return ret_val;
  }




