// C++ program file layer_detect.cpp

/*
  By Nathan C. Hearn
     October 18, 2007
*/


#include <stdlib.h>
#include <iostream>
#include <string>
#include <vector>
#include "mixing_layer.hpp"


const unsigned int buffer_size = 1000;


int main(int argc, char * argv[])
  {
  using std::cout;
  using std::cerr;
  using std::endl;

  const int min_args = 6;

  if (argc < min_args)
    {
    cerr << "Usage: " << argv[0] << " filename min_x max_x bin_spacing "
	 << "poly1 [ poly2 ... ]"
	 << endl << endl;
    return 1;
    }

  int argptr = 1;

  const std::string filename = argv[argptr++];

  const double min_x = atof(argv[argptr++]);
  const double max_x = atof(argv[argptr++]);

  const double bin_spacing = atof(argv[argptr++]);

  std::vector<double> poly_list;

  while (argptr < argc)
    poly_list.push_back(atof(argv[argptr++]));

  const unsigned int num_poly_values = poly_list.size();

  cout << "Poly" << "\t" << "Bubble" << "\t" << "Spike" << endl;

  MixingLayer mix_layer;

  for (unsigned int index = 0; index < num_poly_values; index++)
    {
    const double poly_value = poly_list[index];

    get_mixing_layer(filename, min_x, max_x, bin_spacing, poly_value, 
		     poly_value, mix_layer, buffer_size);

    cout << poly_value << "\t" << mix_layer.get_min_pos() << "\t" 
	 << mix_layer.get_max_pos() << endl;
    }

  cout << endl;
    
  return 0;
  }
