// C++ header file layer_distribution.hpp

/*
  By Nathan C. Hearn
     April 3, 2008

  Container for layer distribution data.
*/


#ifndef LAYER_DISTRIBUTION_HPP
#define LAYER_DISTRIBUTION_HPP


#include <vector>
#include "quickflash_file_datafile.hpp"


class LayerDistribution
  {
  public :

    LayerDistribution();

    LayerDistribution(const QuickFlash::File::DataFile & df, 
		      const unsigned int search_axis,
		      const double sample_bin_width,
		      const double min_coord, const double max_coord,
		      const bool limit_sample_radius=false,
		      const double max_sample_radius=0.0);

    LayerDistribution(const LayerDistribution & source);

    ~LayerDistribution() { }

    LayerDistribution & operator=(const LayerDistribution & source)
      {
      reset(source);
      return *this;
      }

    void reset();

    void reset(const QuickFlash::File::DataFile & df, 
	       const unsigned int search_axis,
	       const double sample_bin_width,
	       const double min_coord, const double max_coord,
	       const bool limit_sample_radius=false,
	       const double max_sample_radius=0.0);

    void reset(const LayerDistribution & source);

    double get_total_mass() const { return total_mass; }
    double get_poly_mass() const { return poly_mass; }
    double get_chbr_mass() const { return chbr_mass; }
    double get_foam_mass() const { return foam_mass; }

    double get_array_min_coord() const { return bin_array_min_coord; }
    double get_array_max_coord() const { return bin_array_max_coord; }

    unsigned int get_num_bins() const { return num_bins; }

    const std::vector<double> & get_bin_centers() const { return bin_centers; }

    const std::vector<double> & get_total_mass_bins() const 
      { return total_mass_bins; }

    const std::vector<double> & get_poly_mass_bins() const 
      { return poly_mass_bins; }

    const std::vector<double> & get_chbr_mass_bins() const 
      { return chbr_mass_bins; }

    const std::vector<double> & get_foam_mass_bins() const 
      { return foam_mass_bins; }

    const std::vector<double> & get_total_dens_variance_bins() const 
      { return total_dens_variance_bins; }

    const std::vector<double> & get_poly_dens_variance_bins() const 
      { return poly_dens_variance_bins; }

    const std::vector<double> & get_chbr_dens_variance_bins() const 
      { return chbr_dens_variance_bins; }

    const std::vector<double> & get_foam_dens_variance_bins() const 
      { return foam_dens_variance_bins; }

  private :

    double total_mass;
    double poly_mass;
    double chbr_mass;
    double foam_mass;

    double bin_array_min_coord;
    double bin_array_max_coord;

    unsigned int num_bins;

    std::vector<double> bin_centers;

    std::vector<double> total_mass_bins;
    std::vector<double> poly_mass_bins;
    std::vector<double> chbr_mass_bins;
    std::vector<double> foam_mass_bins;

    std::vector<double> total_dens_variance_bins;
    std::vector<double> poly_dens_variance_bins;
    std::vector<double> chbr_dens_variance_bins;
    std::vector<double> foam_dens_variance_bins;
  };


#endif // LAYER_DISTRIBUTION_HPP
