// C++ header file part_data_extractor.hpp

/*
  By Nathan C. Hearn
     November 20, 2008

  Class for handling particle extraction from multiple files.
*/


#ifndef PART_DATA_EXTRACTOR_HPP
#define PART_DATA_EXTRACTOR_HPP


#include <vector>
#include <string>
#include "quickflash_file_datafile.hpp"


class PartDataExtractor
  {
  public :

    PartDataExtractor();

    PartDataExtractor(const std::vector<std::string> & file_name_list,
		      const std::vector<std::string> & elem_name_list);

    ~PartDataExtractor() { reset(); }

    void reset();

    void reset(const std::vector<std::string> & file_name_list,
	       const std::vector<std::string> & elem_name_list);

    void add_part_data(const unsigned int particle_tag, 
		       std::vector<double> & times,
		       std::vector< std::vector<double> > & elem_data) const;

  private :

    void setup_elems(const std::vector<std::string> & elem_name_list);

  private :

    unsigned int num_files;

    std::vector<const QuickFlash::File::DataFile *> data_file_list;

    unsigned int num_elems;

    std::vector< std::vector<unsigned int> > file_elem_name_indexes;
  };


#endif  // PART_DATA_EXTRACTOR_HPP
