// C++ program file parts.cpp

/*
  By Nathan C. Hearn
     December 11, 2006

  Demonstration of particle-related functions for the QuickFlash library.
*/


#include <hdf5.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include "quickflash_file_datafile.hpp"
#include "quickflash_particles_data.hpp"


int main(int argc, char * argv[])
  {
  using std::cout;
  using std::cerr;
  using std::endl;

  if (argc < 3)
    {
    cerr << "Usage: " << argv[0] << " filename part_tag" << endl
	 << endl;
    return 1;
    }

  int argptr = 1;

  const std::string file_name = argv[argptr++];

  const unsigned int part_tag = atoi(argv[argptr++]);

  // Open the file

  QuickFlash::File::DataFile dfile(file_name);

  if (!(dfile.particles_present()))
    {
    cerr << "Error: No particle data present" << endl;
    return -2;
    }

  // Get the particle data object

  const QuickFlash::Particles::PartData & partdata = dfile.get_part_data();

  // Output some basic information

  cerr << "Particles : " << partdata.get_num_particles() << endl;

  cerr << "Variables : " << partdata.get_num_variables() << endl;

  const std::vector<std::string> & var_names = partdata.get_variable_names();

  const unsigned int num_variables = var_names.size();

  for (unsigned int index = 0; index < num_variables; index++)
    cout << "  " << index << " : " << var_names[index] << endl;

  // Grab the particle data

  if (!(partdata.particle_present(part_tag)))
    {
    cerr << "Error: Particle not present" << endl;
    return -1;
    }

  std::vector<double> data;

  partdata.get_particle_data_tag(part_tag, data);

  // Print the data

  cout << "Particle tag : " << part_tag << endl;

  const unsigned int num_data = data.size();

  if (num_data != num_variables)
    {
    cerr << "Error: Data vector length mismatch" << endl;
    return -2;
    }

  for (unsigned int index = 0; index < num_data; index++)
    cout << "  " << index << " : " << data[index] << endl;

  return 0;
  }
