// C++ header file pytracer_core.hpp

/*
  By Nathan C. Hearn
     July 19, 2007

  SWIG wrapper class for RaytracerCore.
*/


#ifndef PYTRACER_CORE_HPP
#define PYTRACER_CORE_HPP


#include <string>
#include <vector>
#include "quickflash_file_datafile.hpp"
#include "raytracer_core.hpp"


class PyTracerCore 
  {
  public :
    PyTracerCore() : datafile(), raytracer_core() { }

    PyTracerCore(const std::string & data_file_name,
		 const std::string & density_dataset_name,
		 const double detector_x,
		 const double axis_y, const double axis_z,
		 const unsigned int buffer_size=1000,
		 const unsigned int cache_size=1000) :
      datafile(), raytracer_core()
      { 
      reset(data_file_name, density_dataset_name, detector_x, axis_y, axis_z, 
	    buffer_size, cache_size); 
      }

    ~PyTracerCore() { reset(); }

    void reset()
      {
      raytracer_core.reset();
      datafile.reset();
      }

    void reset(const std::string & data_file_name,
	       const std::string & density_dataset_name,
	       const double detector_x,
	       const double axis_y, const double axis_z,
	       const unsigned int buffer_size=1000,
	       const unsigned int cache_size=1000)
      {
      open_data_file(data_file_name, buffer_size);

      std::vector<double> center(3);

      center[0] = detector_x;
      center[1] = axis_y;
      center[2] = axis_z;

      raytracer_core.reset(datafile, density_dataset_name, center, cache_size);
      }

    void report_cache_stats()
      { raytracer_core.report_cache_stats(); }

    void add_species(const std::string & dataset_name,
		     const double opacity)
      { raytracer_core.add_species(dataset_name, opacity); }

    void set_source_detector(const double axis_azimuth,
			     const double source_distance,
			     const double source_radius,
			     const double sample_spacing,
			     const unsigned int rays_per_pixel,
			     const double film_distance, 
			     const double film_width,
			     const double film_height, 
			     const unsigned int width_pixels,
			     const unsigned int height_pixels)
      {
      raytracer_core.set_source_detector(axis_azimuth, source_distance,
					 source_radius, sample_spacing,
					 rays_per_pixel, film_distance, 
					 film_width, film_height, 
					 width_pixels, height_pixels);
      }

    bool set_up() const { return raytracer_core.set_up(); }

    double get_pixel_value(const unsigned int pixel_index) const
      { return raytracer_core.get_pixel_value(pixel_index); }

  private :
    void open_data_file(const std::string & data_file_name,
			const unsigned int buffer_size)
      { datafile.reset(data_file_name, buffer_size, true); }

  private :
    QuickFlash::File::DataFile datafile;
    RaytracerCore raytracer_core;
  };


#endif  // PYTRACER_CORE_HPP
