#!/usr/bin/env python

# resize_canvas.py

# By Nathan C. Hearn
#    August 24, 2008
#
# Program for setting the geometry of the canvas (but not resizing the image
# itself).


try :
    import psyco
except ImportError :
    pass


def main() :

    from sys import argv, exit, stderr
    from os import path

    from PIL import Image
    
    argc = len(argv)

    min_args = int(5)

    if (argc < min_args) :

        exec_path, exec_name = path.split(argv[0])
        
        stderr.write("\nUsage: " + exec_name + " output_prefix width height "
                     + "file1 [ file2 ... ]\n\n")
        
        exit(1)

    argptr = int(1)

    output_prefix = str(argv[argptr])
    argptr += 1

    output_prefix_path, output_prefix_name = path.split(output_prefix)

    if (len(output_prefix_name) > 0) :
        output_prefix += "_"

    width = int(argv[argptr])
    argptr += 1

    if (width < 1) :
        stderr.write("\nError: Width [ " + str(width) + " ] must be positive"
                     + "\n\n")
        exit(-1)

    height = int(argv[argptr])
    argptr += 1

    if (height < 1) :
        stderr.write("\nError: Height [ " + str(height) + " ] must be positive"
                     + "\n\n")
        exit(-1)

    image_file_list = list()

    while (argptr < argc) :
        image_file_list.append(str(argv[argptr]))
        argptr += 1

    ## Process the files

    for image_file in image_file_list :

        stderr.write("Processing image " + image_file + "\n")

        ## Build the output filename

        image_file_path, image_file_name = path.split(image_file)

        output_file = output_prefix + image_file_name

        ## Load the image

        input_image = Image.open(image_file)

        image_mode = input_image.mode

        ## Determine the offsets

        input_image_width = input_image.size[0]
        input_image_height = input_image.size[1]

        image_offset_horiz = (width - input_image_width) / 2
        image_offset_vert = (height - input_image_height) / 2

        ## Build the output image

        output_image = Image.new(image_mode, (width, height))

        output_image.paste(input_image,
                           (image_offset_horiz, image_offset_vert))

        output_image.save(output_file)


if (__name__ == "__main__") :
    main()

        
