// C++ program file treedemo.cpp

/*
  By Nathan C. Hearn
     December 11, 2006

  Demonstration of tree-related functions in the QuickFlash library, including 
  tree node iterators for Flash data files.
*/


#include <hdf5.h>
#include <string>
#include <vector>
#include <iostream>
#include <stdlib.h>
#include "quickflash_file_datafile.hpp"
#include "quickflash_block_blockinfo.hpp"
#include "quickflash_file_meshinfo.hpp"
#include "quickflash_tree_block.hpp"


int main(int argc, char * argv[])
  {
  using std::cout;
  using std::cerr;
  using std::endl;

  if (argc < 3)
    {
    cerr << "Usage: " << argv[0] << " filename <position>" << endl
	 << endl;
    return 1;
    }

  int argPtr = 1;

  const std::string file_name = argv[argPtr++];

  std::vector<double> pos;

  while (argPtr < argc)
    pos.push_back(atof(argv[argPtr++]));

  // Open the file

  QuickFlash::File::DataFile dfile(file_name);

  // Get simulation info

  const QuickFlash::File::MeshInfo & meshinfo = dfile.get_mesh_info();

  // Look for pos within the tree

  QuickFlash::Tree::ConstTreeIter node_iter = meshinfo.get_tree_iter();

  while (node_iter.valid_node())
    {
    const QuickFlash::Block::BlockInfo & block_info = node_iter.get_block_info();

    if (block_info.in_block(pos))
      {
      const unsigned int block_index = node_iter.get_block_index();
    
      if (node_iter.is_leaf())
	cout << "Position is in leaf node [ " << block_index << " ]" << endl;
      else
	cout << "Position is in branch node [ " << block_index << " ]" << endl;

      node_iter.next();
      }
    else
      node_iter.next_branch();
    }
    
  return 0;
  }
