# tune_gammas/flash_run.py

# By Nathan C. Hearn
#    May 31, 2007
#
# Class for running flash and collecting data


class FlashRun :

    def __init__(self, exec_cmd=None, filename_base="default", gamma_poly=1.4,
                 gamma_foam=1.4, par_template="flash.par.template") :

        from sim_1d import Sim1D

        self.__gamma_poly = float(0.0)
        self.__gamma_foam = float(0.0)

        self.__sim_results = Sim1D()

        self.reset(exec_cmd, filename_base, gamma_poly, gamma_foam,
                   par_template)


    def reset(self, exec_cmd=None, filename_base="default_", gamma_poly=1.4,
              gamma_foam=1.4, par_template="flash.par.template") :

        if (exec_cmd is None) :

            self.__gamma_poly = float(0.0)
            self.__gamma_foam = float(0.0)

            self.__sim_results.reset()

        else :

            self.__run_flash(exec_cmd, filename_base, gamma_poly, gamma_foam,
                             par_template)

            self.__gamma_poly = gamma_poly
            self.__gamma_foam = gamma_foam

            self.__sim_results.reset(filename_base + "hdf5_plt_cnt_")


    def get_interface_pos(self, time) :

        return self.__sim_results.get_interface_pos(time)


    def get_peak_pres_pos(self, time) :

        return self.__sim_results.get_peak_pres_pos(time)


    def __run_flash(self, exec_cmd, filename_base, gamma_poly, gamma_foam,
                    par_template) :

        from sys import stderr
        from os import system

        self.__clean_files(filename_base)

        par_filename = self.__build_parfile(filename_base, gamma_poly,
                                            gamma_foam, par_template)

        # Build the command and execute it

        exec_str = exec_cmd + " -par_file " + par_filename

        if (system(exec_str) != 0) :
            stderr.write("Error: Can not complete command [ " + exec_str
                         + " ]\n")
            raise ValueError("Error: Can not run simulation")


    def __clean_files(self, filename_base) :

        from glob import glob
        from os import system

        old_files = glob(filename_base + "hdf5_*_" + "[0-9][0-9][0-9][0-9]")

        for cur_file in old_files :

            exec_str = "rm -f " + cur_file

            if (system(exec_str) != 0) :
                stderr.write("Error: Can delete file [ " + cur_file + " ]\n")
                raise ValueError("Error: Can not delete file")

    def __build_parfile(self, filename_base, gamma_poly, gamma_foam,
                        par_template) :

        par_filename = filename_base + "flash.par"

        # Read the template

        temp_file = file(par_template, "r")

        file_lines = temp_file.readlines()

        # Write the par file

        par_file = file(par_filename, "w")

        for line in file_lines :

            words = line.split()

            if (len(words) > 0) :

                keyword = words[0].lower()

                if ((keyword != "basenm") and (keyword != "gamma_poly")
                    and (keyword != "gamma_foam")) :

                    par_file.write(line + "\n")

        # Fill in the desired settings

        par_file.write("basenm = " + filename_base + "\n")
        par_file.write("gamma_poly = " + str(gamma_poly) + "\n")
        par_file.write("gamma_foam = " + str(gamma_foam) + "\n")

        return par_filename

