# tune_gammas/read_sim.py

# By Nathan C. Hearn
#    May 31, 2007
#
# Routines for extracting interface and shock-front positions from 1D
# Flash RT/RM simulations


target_foam = float(0.5)

foam_name = str("foam")
pres_name = str("pres")


def read_file(filename) :

    import quickflash

    # Open the file and get basic information

    df = quickflash.Datafile(filename, 1000)

    siminfo = df.get_sim_info()

    sim_time = siminfo.get_sim_time()

    dims = siminfo.get_dims()

    if (dims < 1) :
        raise ValueError("Insufficient spatial dimensions")

    num_blocks = siminfo.get_num_blocks()

    min_x = siminfo.get_volume_minbounds()[0]

    # Load the datasets

    foam_set = df.get_dataset(foam_name)
    pres_set = df.get_dataset(pres_name)

    # Parse the leaf blocks

    peak_pres_pos = min_x
    peak_pres_val = 0.0  # Assume only positive pressures

    interface_pos = min_x

    coords = quickflash.DoubleVect()

    foam_data = quickflash.DoubleBlock()
    pres_data = quickflash.DoubleBlock()

    for block_index in range(num_blocks) :
        
        if (siminfo.is_leaf(block_index)) :

            blockinfo = siminfo.get_block_info(block_index)

            num_cells = blockinfo.get_num_cells()

            foam_set.get_block_data(block_index, foam_data)
            pres_set.get_block_data(block_index, pres_data)

            for cell_index in range(num_cells) :

                blockinfo.get_cell_center(cell_index, coords)

                x_coord = coords[0]

                foam_val = foam_data[cell_index]
                pres_val = pres_data[cell_index]

                if (pres_val > peak_pres_val) :
                    peak_pres_pos = x_coord
                    peak_pres_val = pres_val

                if (not (foam_val > target_foam)) :
                    if (x_coord > interface_pos) :
                        interface_pos = x_coord

    return sim_time, interface_pos, peak_pres_pos


def read_sim(filename_base) :

    from glob import glob
    from numarray import zeros, Float

    # Get the filenames

    filename_list = glob(filename_base + "[0-9][0-9][0-9][0-9]")

    num_files = len(filename_list)

    if (num_files < 1) :
        raise ValueError("Error: No files found")

    time_list = zeros(num_files, Float)
    interface_pos_list = zeros(num_files, Float)
    peak_pres_pos_list = zeros(num_files, Float)

    for index in range(num_files) :

        time, interface_pos, peak_pres_pos = read_file(filename_list[index])

        time_list[index] = time
        interface_pos_list[index] = interface_pos
        peak_pres_pos_list[index] = peak_pres_pos

    return time_list, interface_pos_list, peak_pres_pos_list
