# tune_gammas/sim_1d.py

# By Nathan C. Hearn
#    May 31, 2007
#
# Container class for info from 1D RT/RM simulations


class Sim1D :

    def __init__(self, filename_base=None) :

        self.__num_times = int(0)

        self.__min_time = float(0.0)
        self.__max_time = float(0.0)

        self.__time_list = None
        self.__interface_list = None
        self.__peak_pres_list = None

        self.reset(filename_base)


    def reset(self, filename_base=None) :

        from read_sim import read_sim

        if (filename_base is None) :
            
            self.__num_times = int(0)
            self.__time_list = None
            self.__interface_list = None
            self.__peak_pres_list = None
            
        else :
            
            self.__time_list, self.__interface_list, self.__peak_pres_list \
                              = read_sim(filename_base)

            self.__num_times = len(self.__time_list)

            # DO WE NEED TO WORRY ABOUT SORTING???

            self.__min_time = self.__time_list[0]
            self.__max_time = self.__time_list[self.__num_times - 1]


    def get_interface_pos(self, time) :

        lower_index, upper_index, lower_weight, upper_weight \
                     = self.__get_interp_time(time)

        lower_value = self.__interface_list[lower_index]
        upper_value = self.__interface_list[upper_index]

        return (lower_value * lower_weight) + (upper_value * upper_weight)


    def get_peak_pres_pos(self, time) :

        lower_index, upper_index, lower_weight, upper_weight \
                     = self.__get_interp_time(time)

        lower_value = self.__peak_pres_list[lower_index]
        upper_value = self.__peak_pres_list[upper_index]

        return (lower_value * lower_weight) + (upper_value * upper_weight)


    def __get_interp_time(self, time) :

        if (self.__num_times < 2) :
            raise ValueError("Must have at least two times for interpolation")

        if ((time < self.__min_time) or (time > self.__max_time)) :
            raise ValueError("Supplied time is out of range")

        lower_index = int(0)
        upper_index = int(self.__num_times - 1)

        while (lower_index < (upper_index - 1)) :

            test_index = int((lower_index + upper_index) / 2)

            test_time = self.__time_list[test_index]

            if (test_time > time) :
                upper_index = test_index
            else :
                lower_index = test_index

        # Determine weights

        lower_time = self.__time_list[lower_index]
        upper_time = self.__time_list[upper_index]

        interval_time = upper_time - lower_time

        if (not (interval_time > 0.0)) :
            raise ValueError("Non-positive time interval between files")

        delta_time_lower_time = time - lower_time

        upper_weight = delta_time_lower_time / interval_time

        if (upper_weight < 0.0) :
            upper_weight = 0.0
        elif (upper_weight > 1.0) :
            upper_weight = 1.0

        lower_weight = 1.0 - upper_weight

        return lower_index, upper_index, lower_weight, upper_weight

    

        

        

