// C++ program file cubic_volume.cpp

/*
  By Nathan C. Hearn
     May 28, 2007

  Tools for detecting overlap fraction between two rectangular volumes.
*/


#include "cubic_volume.hpp"
#include <vector>


bool volumes_overlap(const std::vector<double> & primary_min,
		     const std::vector<double> & primary_max,
		     const std::vector<double> & secondary_min,
		     const std::vector<double> & secondary_max)
  {
  double overlap_volume = 0.0;
  std::vector<double> overlap_center;
  double primary_fraction = 0.0;
  double secondary_fraction = 0.0;

  return volumes_overlap(primary_min, primary_max, secondary_min, 
			 secondary_max, overlap_volume, overlap_center,
			 primary_fraction, secondary_fraction);
  }


bool volumes_overlap(const std::vector<double> & primary_min,
		     const std::vector<double> & primary_max,
		     const std::vector<double> & secondary_min,
		     const std::vector<double> & secondary_max,
		     double & overlap_volume, 
		     std::vector<double> & overlap_center,
		     double & primary_fraction, double & secondary_fraction)
  {
  bool ret_val = false;  // Default

  const unsigned int dims = primary_min.size();

  // ERROR CHECKING????

  // Get overlap

  std::vector<double> center_coords(dims, 0.0);

  bool overlapping = true;

  double primary_volume = 1.0;
  double secondary_volume = 1.0;

  double volume = 1.0;

  for (unsigned int i = 0; i < dims; i++)
    {
    const double primary_min_coord = primary_min[i];
    const double primary_max_coord = primary_max[i];

    const double secondary_min_coord = secondary_min[i];
    const double secondary_max_coord = secondary_max[i];

    const double overlap_min_coord 
      = (primary_min_coord > secondary_min_coord) 
        ? primary_min_coord : secondary_min_coord;

    const double overlap_max_coord 
      = (primary_max_coord < secondary_max_coord) 
        ? primary_max_coord : secondary_max_coord;

    if (!(overlap_min_coord < overlap_max_coord))
      overlapping = false;

    // Get volumes of original domains

    primary_volume *= primary_max_coord - primary_min_coord;
    secondary_volume *= secondary_max_coord - secondary_min_coord;

    // Get the volume and center position of the overlap

    volume *= overlap_max_coord - overlap_min_coord;

    center_coords[i] = 0.5 * (overlap_min_coord + overlap_max_coord);
    }

  if (overlapping)
    {
    overlap_volume = volume;
    overlap_center = center_coords;

    // Get volume fractions

    const double oneOver_overlap_volume = 1.0 / overlap_volume;

    primary_fraction = primary_volume * oneOver_overlap_volume;
    secondary_fraction = secondary_volume * oneOver_overlap_volume;
    }
  else
    {
    overlap_volume = 0.0;
    overlap_center.clear();

    primary_fraction = 0.0;
    secondary_fraction = 0.0;
    }

  ret_val = overlapping;

  return ret_val;
  }


bool volume_enclosed(const std::vector<double> & primary_min,
		     const std::vector<double> & primary_max,
		     const std::vector<double> & sample_min,
		     const std::vector<double> & sample_max)
  {
  bool ret_val = false;  // Default

  const unsigned int dims = primary_min.size();

  // ERROR CHECKING????  RIGHT NOW, WE ASSUME INPUT ELEMENTS MIN < MAX

  // Get overlap

  std::vector<double> center_coords(dims, 0.0);

  bool enclosed = true;

  for (unsigned int i = 0; i < dims; i++)
    {
    const double primary_min_coord = primary_min[i];
    const double primary_max_coord = primary_max[i];

    const double sample_min_coord = sample_min[i];
    const double sample_max_coord = sample_max[i];

    if ((sample_min_coord < primary_min_coord) 
	|| (sample_max_coord > primary_max_coord))
      enclosed = false;
    }

  ret_val = enclosed;

  return ret_val;
  }
