# read_table.py

# By Nathan C. Hearn
#    October 16, 2007
#
# Loading of tables from file


def read_line(file_object) :

    ret_val = None

    reached_eof = False
    found_line = False

    while ((not reached_eof) and (not found_line)) :

        line = file_object.readline()

        if (line == "") :
            
            reached_eof = True
            
        else :

            no_comment_line = line.split("#")[0].strip()

            if (no_comment_line != "") :
                
                ret_val = no_comment_line

                found_line = True

    return ret_val


def read_table(filename) :

    f = file(filename, "r")

    # Read the headings

    heading_line = read_line(f)

    if (heading_line is None) :
        raise RuntimeError("Unable to find headings in file " + filename)

    headings = heading_line.split()

    num_columns = len(headings)

    data = list()

    for i in range(num_columns) :
        data.append(list())

    # Load the data

    end_reached = False

    while (not end_reached) :

        data_line = read_line(f)

        if (data_line is None) :

            end_reached = True

        else :

            data_elements = data_line.split()

            if (len(data_elements) != num_columns) :
                raise RuntimeError("Column-count mismatch")

            for i in range(num_columns) :
                data[i].append(float(data_elements[i]))

    # Construct the table

    table = dict()

    for i in range(num_columns) :

        column = data[i]

        column_heading = headings[i]

        if (table.has_key(column_heading)) :
            raise RuntimeError("Duplicate column heading")

        table[column_heading] = column

    return table
