// C++ header file sim_1d.hpp

/*
  By Nathan C. Hearn
     June 1, 2007

  Tools for extracting interface position from 1D simulations.
*/


#ifndef SIM_1D_HPP
#define SIM_1D_HPP


#include <string>
#include <vector>



// Helper functions

void read_file(const std::string & filename, double & sim_time, 
	       double & interface_pos);

void read_sim(const std::string & filename_base, 
	      std::vector<double> & time_list, 
	      std::vector<double> & interface_pos_list);


// Class Sim1D

class Sim1D
  {
  public:

    Sim1D() : 
      num_times(0), min_time(0.0), max_time(0.0), time_list(), interface_list()
      { }

    Sim1D(const std::string & filename_base) : 
      num_times(0), min_time(0.0), max_time(0.0), time_list(), interface_list()
      { reset(filename_base); }

    Sim1D(const Sim1D & source) : 
      num_times(0), min_time(0.0), max_time(0.0), time_list(), interface_list()
      { reset(source); }

    ~Sim1D() { }

    Sim1D & operator=(const Sim1D & source)
      {
      reset(source);
      return *this;
      }

    void reset();

    void reset(const std::string & filename_base);

    void reset(const Sim1D & source);

    void get_interface_pos(const double time, double & interface_pos,
			   double & interface_vel);

  private:
    int get_interp_time(const double time, unsigned int & lower_index,
			unsigned int & upper_index, double & lower_weight,
			double & upper_weight);

  private:
    unsigned int num_times;

    double min_time;
    double max_time;

    std::vector<double> time_list;
    std::vector<double> interface_list;
  };



#endif  // SIM_1D_HPP
