// C++ header file blockdata.hpp

/*
  By Nathan C. Hearn
     November 20, 2006

  SWIG wrapper for QuickFlash::Block::BlockData template.
*/


#ifndef BLOCKDATA_HPP
#define BLOCKDATA_HPP


#include <vector>
#include "../src_lib/quickflash_block_blockdata.hpp"


template<typename T>
class BlockData
  {
  public:
    BlockData() : data() { }

    BlockData(const std::vector<unsigned int> & dims) :
      data()
      { reset(dims); }

    BlockData(const QuickFlash::Block::BlockData<T> & source) : 
      data() 
      { reset(source); }

    BlockData(const BlockData<T> & source) : data() { reset(source); }

    ~BlockData() { }

    void reset(const BlockData<T> & source)
      {
      if (&source != this)
	data = source.data;
      }

    void reset(const std::vector<unsigned int> & dims)
      { data.reset(dims); }

    void reset(const QuickFlash::Block::BlockData<T> & source)
      { data = source; }

    unsigned int size() const { return data.get_num_cells(); }

    unsigned int __len__() const { return data.get_num_cells(); }

    T & operator[](const unsigned int index) { return data[index]; }

    const T & operator[](const unsigned int index) const 
      { return data[index]; }

    void __setitem__(const unsigned int index, const T & value) 
      { data[index] = value; }

    const T & __getitem__(const unsigned int index) const
      { return data[index]; }

    QuickFlash::Block::BlockData<T> & get_ref() { return data; }
    const QuickFlash::Block::BlockData<T> & get_ref() const { return data; }

  private:
    QuickFlash::Block::BlockData<T> data;
  };


#endif  // BLOCKDATA_HPP
