# read_grid_datafile.py

# By Nathan C. Hearn
#    June 23, 2008
#
# Grid datafile reader


def remove_comments(line) :

    # Remove comment characters "#" and all following characters

    clean_line = line.split("#")[0].strip()

    return clean_line


def read_line(input_stream) :

    at_eof = False

    next_line = None

    while ((not at_eof) and (next_line is None)) :

        line = input_stream.readline()

        if (line == "") :

            at_eof = True

        else :

            clean_line = remove_comments(line.strip())

            if (clean_line != "") :
                next_line = clean_line

    return next_line


def read_file(filename, row_range=None, col_range=None,
              data_start_marker=None) :

    from numeric_defs import numeric_zeros, numeric_float

    # NOTE: The data is read in row-major format.

    rows = list()

    f = file(filename, "r")

    cols_counted = False

    num_cols = int(0)

    at_eof = False

    found_data_start = False

    if (data_start_marker is None) :
        found_data_start = True

    while (not at_eof) :

        line = read_line(f)

        if (line is None) :

            at_eof = True

        else :

            if (len(line) > 0) :

                if (not found_data_start) :

                    if (line == data_start_marker) :
                        found_data_start = True

                else :

                    words = line.split()

                    cols = list()

                    for num in words :
                        cols.append(float(num))

                    if (cols_counted) :
                        if (len(cols) != num_cols) :
                            raise ValueError("Incorrect number of columns")
                    else :
                        num_cols = len(cols)
                        cols_counted = True

                    rows.append(cols)

    num_rows = len(rows)

    num_selected_rows = num_rows
    num_selected_cols = num_cols

    selected_rows_start = 0
    selected_cols_start = 0

    if (row_range is not None) :

        if (len(row_range) != 2) :
            raise RuntimeError("Improper row range list")

        row_start = row_range[0]
        row_end = row_range[1] + 1  ## One past the number requested

        if (row_start < 0) :
            raise RuntimeError("Row start out of range")

        if (row_end > num_rows) :
            raise RuntimeError("Row end out of range")

        if (row_start >= row_end) :
            raise RuntimeError("Row start greater than row end")

        selected_rows_start = row_start
        num_selected_rows = row_end - row_start

    if (col_range is not None) :

        if (len(col_range) != 2) :
            raise RuntimeError("Improper column range list")

        col_start = col_range[0]
        col_end = col_range[1] + 1  ## One past the number requested

        if (col_start < 0) :
            raise RuntimeError("Column start out of range")

        if (col_end > num_cols) :
            raise RuntimeError("Column end out of range")

        if (col_start >= col_end) :
            raise RuntimeError("Column start greater than col end")

        selected_cols_start = col_start
        num_selected_cols = col_end - col_start


    data = numeric_zeros((num_selected_rows, num_selected_cols), numeric_float)

    for i in range(num_selected_rows) :

        row_list = rows[selected_rows_start + i]
        
        for j in range(num_selected_cols) :

            data[i][j] = row_list[selected_cols_start + j]

    return data
