# read_image_data.py

# By Nathan C. Hearn
#    June 24, 2008
#
# Routines for reading images as grid data


def read_image(filename) :

    from PIL import Image

    return Image.open(filename)


def image_data(image, row_range=None, col_range=None) :

    from numeric_defs import numeric_zeros, numeric_float

    gray_image = image.convert("L")

    image_size = gray_image.size

    image_i_size = image_size[0]
    image_j_size = image_size[1]

    i_start = 0
    i_end = image_i_size  ## One past last element

    j_start = 0
    j_end = image_j_size  ## One past last element

    if (row_range is not None) :
        i_start = row_range[0]
        i_end = row_range[1] + 1
    
    if (col_range is not None) :
        j_start = col_range[0]
        j_end = col_range[1] + 1
    
    i_size = i_end - i_start
    j_size = j_end - j_start

    image_pixels = gray_image.load()

    data = numeric_zeros((i_size, j_size), numeric_float)

    for i in range(i_size) :

        image_i = i + i_start

        image_i_in_range = False

        if ((image_i >= 0) and (image_i < image_i_size)) :
            image_i_in_range = True
        
        for j in range(j_size) :

            image_j = j + j_start

            value = float(0.0)  ## Default value

            if (image_i_in_range) :
                if ((image_j >= 0) and (image_j < image_j_size)) :
                    value = float(image_pixels[image_i, image_j])

            data[i][j] = value

    return data


def read_image_data(filename, row_range=None, col_range=None) :

    image = read_image(filename)

    return image_data(image, row_range, col_range)
