# keyword_values.py

# By Nathan C. Hearn
#    October 10, 2008
#
# Container for a list of keyword-value pairs


class KeywordValuePair :

    def __init__(self, keyword=None, value=None) :

        self.__keyword = None
        self.__value = None

        self.reset(keyword, value)
        

    def reset(self, keyword=None, value=None) :

        self.__keyword = None

        if (keyword is not None) :
            self.__keyword = str(keyword)

        self.__value = None

        if (value is not None) :
            self.__value = str(value)  ## Type???
            

    def __cmp__(self, other) :

        ## Sort based on keywords -- missing keywords have lower value

        ret_val = 0  ## Equal if neither have keywords

        self_keyword = self.get_keyword()

        other_keyword = None

        try :
            other_keyword = other.get_keyword()
        except :
            other_keyword = str(other)

        if ((self_keyword is not None) and (other_keyword is not None)) :
            ret_val = cmp(self_keyword, other_keyword)            
        elif (self_has_keyword) :
            ret_val = 1  ## Missing keyword has less value
        elif (other_has_keyword) :
            ret_val = -1  ## Missing keyword has less value

        ## CAN WE COMPARE VALUES (DIFFERENT TYPES?) IF NO KEYWORDS???

        return ret_val
        

    def reset_keyword(self, keyword=None) :

        if (keyword is not None) :
            self.__keyword = str(keyword)
        else :
            self.__keyword = None

    def get_keyword(self) :
        return self.__keyword

    def keyword_present(self) :

        ret_val = False

        if (self.__keyword is not None) :
            ret_val = True

        return ret_val

    def reset_value(self, value) :

        if (value is not None) :
            self.__value = str(value)
        else :
            self.__value = None

    def get_value(self) :
        return self.__value

    def value_present(self) :

        ret_val = False

        if (self.__value is not None) :
            ret_val = True

        return ret_val

    def get_value_int(self) :
        return int(self.__value)

    def get_value_float(self) :
        return float(self.__value)

    def get_value_str_list(self, element_separator=",") :
        return self.__value.split(",")

    def get_value_int_list(self, element_separator=",") :

        str_list = self.get_value_str_list(element_separator)

        ret_list = list()

        for elem in str_list :
            ret_list.append(int(elem))

        return ret_list

    def get_value_float_list(self, element_separator=",") :

        str_list = self.get_value_str_list(element_separator)

        ret_list = list()

        for elem in str_list :
            ret_list.append(float(elem))

        return ret_list


class KeywordValueList :

    def __init__(self) :

        self.__keyword_value_list = list()

    def reset(self) :

        self.__keyword_value_list = list()

    def __len__(self) :
        return len(self.__keyword_value_list)

    def __getitem__(self, index) :
        return self.__keyword_value_list[index]

    def get_num_pairs(self) :
        return len(self.__keyword_value_list)

    def add_keyword_value(self, keyword, value=None) :

        new_pair = KeywordValuePair(keyword, value)

        ## For now, use insertion sort to keep list sorted

        old_num_elems = self.get_num_pairs()

        index = 0

        found_pos = False

        while ((not found_pos) and (index < old_num_elems)) :

            if (cmp(new_pair, self.__keyword_value_list[index]) < 1) :
                found_pos = True
            else :
                index += 1

        if (not found_pos) :
            self.__keyword_value_list.append(new_pair)
        else :
            self.__keyword_value_list.insert(index, new_pair)  ## Before index

    def get_keyword_value_list(self) :
        return self.__keyword_value_list

    def find_first_pair(self, keyword) :

        ret_val = None

        keyword_str = str(keyword)

        ## For now, just use linear search to get first pair with this keyword

        num_pairs = self.get_num_pairs()

        index = 0

        while ((ret_val is None) and (index < num_pairs)) :

            comp_keyword = self.__keyword_value_list[index].get_keyword()
            
            if (keyword_str == comp_keyword) :
                ret_val = index
            else :
                index += 1

        return ret_val

    def get_first_value(self, keyword) :

        ret_val = None

        index = self.find_first_pair(keyword)

        if (index is not None) :
            ret_val = self.__keyword_value_list[index].get_value()

        return ret_val
