// C++ header file star.hpp

/*
  By Nathan C. Hearn
     June 14, 2008

  Star element prototype.
*/

#ifndef STAR_HPP
#define STAR_HPP


#include "quickflash_color_color.hpp"
#include "quickflash_color_colormap.hpp"


class Star
  {
  public :

    Star();

    Star(const double center_x, const double center_y, 
	 const double radial_scale, const double max_radius,
	 const QuickFlash::Color::Color 
	   & star_color=QuickFlash::Color::Color(1.0),
	 const unsigned int num_spikes=0, const double spike_scale_factor=4.0,
	 const double spike_phase_deg=0.0, 
	 const double spike_steepening_power=2.0);

    Star(const Star & source);

    ~Star() { }

    Star & operator=(const Star & source)
      {
      reset(source);
      return *this;
      }

    void reset();

    void reset(const double center_x, const double center_y, 
	       const double radial_scale, const double max_radius,
	       const QuickFlash::Color::Color 
	         & star_color=QuickFlash::Color::Color(1.0), 
	       const unsigned int num_spikes=0, 
	       const double spike_scale_factor=0.0,
	       const double spike_phase_deg=0.0, 
	       const double spike_steepening_power=4.0);
	
    void reset(const Star & source);

    void set_pos(const double center_x, const double center_y)
      {
      star_x = center_x;
      star_y = center_y;
      }

    void set_size(const double radial_scale, const double max_radius);

    void set_spikes(const unsigned int num_spikes=0, 
		    const double spike_scale_factor=0.0,
		    const double spike_phase_deg=0.0,
		    const double spike_steepening_power=4.0);
    
    void set_star_color(const QuickFlash::Color::Color & star_color);

    double get_radial_scale() const { return rad_scale; }

    double get_max_radius() const { return max_rad; }
    double get_sq_max_radius() const { return sq_max_rad; }

    bool get_color(const double x, const double y, 
		   QuickFlash::Color::Color & pos_color) const;

  private :

    double get_spike_factor(const double rel_x, const double rel_y) const;
    double get_value(const double rel_x, const double rel_y) const;

  private :

    static const double deg_to_rad;

    double star_x;
    double star_y;

    double rad_scale;
    double oneOver_sq_rad_scale;

    double max_rad;
    double sq_max_rad;

    bool use_spikes;

    double spike_angle_freq;
    double spike_scale_fact;
    double spike_phase_rad;
    double spike_power;

    QuickFlash::Color::Colormap colormap;
  };





#endif  // STAR_HPP
