// C++ header file star_generator.hpp

/*
  By Nathan C. Hearn
     June 14, 2008

  Generator for Star objects.
*/


#ifndef STAR_GENERATOR_HPP
#define STAR_GENERATOR_HPP


#include "star.hpp"
#include "random_value.hpp"


class StarGenerator
  {
  public :

    StarGenerator();

    StarGenerator(const double image_width, const double image_height, 
		  const double min_radial_scale, const double max_radial_scale,
		  const double max_radius_factor, 
		  const double min_saturation, const double max_saturation, 
		  const double min_brightness, const double max_brightness,
		  const unsigned int num_spikes=0, 
		  const double spike_scale_factor=4.0, 
		  const double spike_phase_deg=0.0, 
		  const double spike_steepening_power=4.0);

    StarGenerator(const StarGenerator & source);

    ~StarGenerator() { }

    StarGenerator & operator=(const StarGenerator & source)
      {
      reset(source);
      return *this;
      }

    void reset();

    void reset(const double image_width, const double image_height, 
	       const double min_radial_scale, const double max_radial_scale,
	       const double max_radius_factor, 
	       const double min_saturation, const double max_saturation, 
	       const double min_brightness, const double max_brightness,
	       const unsigned int num_spikes=0, 
	       const double spike_scale_factor=4.0, 
	       const double spike_phase_deg=0.0, 
	       const double spike_steepening_power=4.0);

    void reset(const StarGenerator & source);

    void get_star(Star & star, const bool protect_border=true) const;

  private :
    
    double width;
    double height;

    RandomValue x_pos_gen;
    RandomValue y_pos_gen;

    RandomValue radial_scale_gen;

    double max_radius_fact;

    RandomValue hue_gen;
    RandomValue saturation_gen;
    RandomValue brightness_gen;

    unsigned int spike_count;
    double spike_scale;
    double spike_phase;
    double spike_power;
  };


#endif // STAR_GENERATOR_HPP
